package org.saev.deob.trans.pred;

import org.saev.deob.Application;
import org.saev.deob.util.FieldID;
import java.util.*;
import org.apache.bcel.classfile.*;
import org.apache.bcel.generic.*;
import org.apache.bcel.util.*;

public class CPControlFinder implements ControlFinder {
	private static final boolean DEBUG = false;
	private Set<FieldID> fieldSet;
	private FieldID mainField;
		
	public CPControlFinder(Application app) {
		ClassGen mainClass = app.getMainClass();
			
		if(mainClass == null) {
			throw new RuntimeException("error: could not find main class");
		}
		ConstantPoolGen pool = mainClass.getConstantPool();
		Constant c = pool.getConstant(pool.getSize() - 1);
			
		if(!(c instanceof ConstantFieldref)) {
			throw new RuntimeException("error: constant at index " + (pool.getSize() - 1) + " is not a fieldref");
		}
		ConstantFieldref cfr = (ConstantFieldref) pool.getConstant(pool.getSize() - 1);
		ConstantNameAndType cnat = (ConstantNameAndType) pool.getConstant(cfr.getNameAndTypeIndex());
		ConstantPool cp = pool.getConstantPool();
		mainField = new FieldID(cnat.getName(cp), cnat.getSignature(cp), cfr.getClass(cp));
			
		if(DEBUG) {
			System.out.println("main field = " + mainField);
		}
		
		for(Iterator<ClassGen> classIt = app.getClasses().iterator(); classIt.hasNext(); ) {
			ClassGen cgen = classIt.next();
			Method[] methods = cgen.getMethods();
			final String className = cgen.getClassName();
			final ConstantPoolGen cpg = cgen.getConstantPool();
			
			for(int i = 0; i < methods.length; i++) {
				MethodGen mg = new MethodGen(methods[i], className, cpg);
				InstructionList il = mg.getInstructionList();
				
				Iterator<InstructionHandle[]> matchIt =
						new InstructionFinder(il).search("GETSTATIC IFEQ ((IINC ILOAD) | (ILOAD IFEQ ICONST GOTO ICONST)) PUTSTATIC",
						new InstructionFinder.CodeConstraint() {
						
					public boolean checkCode(InstructionHandle[] match) {
						FieldInstruction fi = (FieldInstruction) match[0].getInstruction();
						FieldID fid = new FieldID(fi.getName(cpg), fi.getSignature(cpg), fi.getClassName(cpg));
				
						if(fid.equals(mainField)) {
							return true;
						}
						return false;
					}
				});
					
				while(matchIt.hasNext()) {
					InstructionHandle[] match = matchIt.next();
					FieldInstruction fi = (FieldInstruction) match[match.length - 1].getInstruction();
				
					if(fieldSet == null) {
						fieldSet = new HashSet<FieldID>();
					}
					FieldID fid = new FieldID(fi.getName(cpg), fi.getSignature(cpg), fi.getClassName(cpg));
					fieldSet.add(fid);
						
					if(DEBUG) {
						System.out.println("field = " + fid);
					}
				}
			}
		}
	}
		
	public boolean containsField(FieldID fid) {
		return fieldSet.contains(fid);
	}
		
	public FieldID getControlField() {
			return mainField;
	}
}