
package org.saev.deob.util;

import org.apache.bcel.generic.FieldGen;

/** a wrapper class for FieldGen objects. */
public class FieldID {
    
    private String name;
    private String signature;
    private String className;

	/** creates a FieldID object.
	 * @param name the field name.
	 * @param signature the field signature.
	 * @param className the name of the class this field belongs to. */ 
    public FieldID(String name, String signature, String className) {
        this.name = name;
        this.signature = signature;
        this.className = className;
    }

	/** creates a FieldID objects.
	 * @param fg a FieldGen object. */
    public FieldID(FieldGen fg) {
        this.name = fg.getName();
        this.signature = fg.getSignature();
        this.className = fg.getClassName();
    }
    
    public boolean equals(Object o) {
        
        if(o instanceof FieldID) {
            FieldID fid = (FieldID) o;
            return name.equals(fid.name) && 
                    signature.equals(fid.signature) && 
                    className.equals(fid.className);
        }
        return false;
    }
    
    public int hashCode() {
        return  name.hashCode() + signature.hashCode() + className.hashCode();
    }

	/** gets the name of this FieldID.
	 * @return the name of this FieldID. */ 
    public String getName() {
        return name;
    }

	/** get the signature of this FieldID.
	 * @return the signature of this FieldID. */
    public String getSignature() {
        return signature;
    }

	/** gets the class name of this FieldID.
	 * @return the class name of this FieldID. */
    public String getClassName() {
        return className;
    }

	/** gets a string representation of this FieldID.
	 * @return a string representation of this FieldID. */
    public String toString() {
        return toString(true);
    }

	/** gets a string representation of this FieldID.
	 * @param flag if true, the output matches <code>toString()</code>,
	 * else if false, the output is shorter.
	 * @return a string representation of this FieldID. */
    public String toString(boolean flag) {
        return flag ? "FIELD[" + className + "." + name + "." + signature + "]" :
            className + "." + name + "." + signature;
    }
}