
package org.saev.deob.util;

import org.apache.bcel.generic.MethodGen;
/** a wrapper class for MethodGen objects. */
public class MethodID {
    
    private String name;
    private String signature;
    private String className;

	/** creates a MethodID object.
	 * @param name the method name.
	 * @param signature the method signature.
	 * @param className the name of the class this method belongs to. */
    public MethodID(String name, String signature, String className) {
        this.name = name;
        this.signature = signature;
        this.className = className;
    }

	/** creates a MethodID objects.
	 * @param mg a MethodGen object. */
    public MethodID(MethodGen mg) {
        this.name = mg.getName();
        this.signature = mg.getSignature();
        this.className = mg.getClassName();
    }
    
    public boolean equals(Object o) {
        
        if(o instanceof MethodID) {
            MethodID mid = (MethodID) o;
            return name.equals(mid.name) && 
                    signature.equals(mid.signature) && 
                    className.equals(mid.className);
        }
        return false;
    }
    
    public int hashCode() {
        return  name.hashCode() + signature.hashCode() + className.hashCode();
    }
    
	/** gets the name of this MethodID.
	 * @return the name of this MethodID. */
    public String getName() {
        return name;
    }

	/** get the signature of this MethodID.
	 * @return the signature of this MethodID. */
    public String getSignature() {
        return signature;
    }
    
	/** gets the class name of this MethodID.
	 * @return the class name of this MethodID. */
    public String getClassName() {
        return className;
    }

	/** gets a string representation of this MethodID.
	 * @return a string representation of this MethodID. */
    public String toString() {
        return toString(true);
    }

	/** gets a string representation of this MethodID.
	 * @param flag if true, the output matches <code>toString()</code>,
	 * else if false, the output is shorter.
	 * @return a string representation of this MethodID. */
    public String toString(boolean flag) {
        return flag ? "METHOD[" + className + "." + name + "." + signature + "]" :
            className + "." + name + "." + signature;
    }
}
